local WTN = LibStub("AceAddon-3.0"):NewAddon("GatherNow", "AceEvent-3.0")

local DEFAULT_NOTIFY_SOUND = "Sound\\Spells\\Spell_TimePortal_Teleport.ogg"
local EMPTY_TEXTURE = "Interface\\DialogFrame\\UI-DialogBox-Background"
local WTNframe,fontstring,fontstring2,fontstring3,bR1,teR1,bR2,teR2,bR3,teR3

local _,MYFACTION,MYCONTINENT,isTracking,MsgTrack
local prof1,prof2,archaeology,fishing,cooking,firstAid,Herbalism,Mining,TrackName
local name,icon,skillLevel,maxSkillLevel,numAbilities,spelloffset,skillLine,skillModifier
local Rname,link,quality,iLevel,reqLevel,class,subclass,maxStack,equipSlot,texture,vendorPrice

local BUTTON1_HORZ_OFFSET,BUTTON1_VERT_OFFSET = -24,	10
local BUTTON2_HORZ_OFFSET,BUTTON2_VERT_OFFSET =   0,	10
local BUTTON3_HORZ_OFFSET,BUTTON3_VERT_OFFSET =  24,	10
local RBUTTON_WIDTH,RBUTTON_HEIGHT,PBUTTON_WIDTH,PBUTTON_HEIGHT	= 20, 20, 24, 24 -- resource/profession button size

local MINING_SKILLID,HERBALISM_SKILLID = 186,182
local MINING_NAME,HERBALISM_NAME,MAX_PROFESSION_SKILL = "",""
local localizedContinents,localizedRankData,localizedMapData = {},{},nil

-- ACE3 Standard functions------------------------------------------------------
-- Grabbed need information on init
local defaults = {
	point = "CENTER",
	relativeTo = UIParent,
	relativePoint = "CENTER",
	x = 0,
	y = 0,
	loc = {},
}
function WTN:OnInitialize()
	MYFACTION, _ = UnitFactionGroup("player")
	isTracking = false
	Rname = "" -- initialize empty string
	GameTooltip:SetText("")
	skillLevel = ""
	maxSkillLevel = ""
	MsgTrack = ""
	GatherNowDB = GatherNowDB or CopyTable(defaults)
	if not GatherNowDB.loc then GatherNowDB.loc = {} end
	localizedMapData = GatherNowDB.loc
	if not GatherNowDB.dbver then
		GatherNowDB.loc = wipe(GatherNowDB.loc or {})
		GatherNowDB.dbver = 2
	end
	-- metatables can give us events for read (__index) or write (__newindex) access to a table
	-- allowing us to dymanically return data for missing keys and optionally save them back to the table.
	setmetatable(localizedMapData, 
	{__index = function(t,k) -- this function only runs when we try to read a key from localizedMapData that doesn't yet exist
		if WorldMapFrame:IsShown() then return end
		local name = GetMapNameByID(k)
		if not name then return end
		local oldID = GetCurrentMapAreaID()
		SetMapByID(k)
		local minL,maxL = GetCurrentMapLevelRange()
		local numCont = GetCurrentMapContinent()
		local pvpType = GetZonePVPInfo()
		SetMapByID(oldID)
		local nameCont = numCont and localizedContinents[numCont]
		if minL and maxL then
			if minL ~= maxL then
				name = ("%s (%d-%d) %s"):format(name,minL,maxL,nameCont or "")
			else
				name = ("%s (%d) %s"):format(name,maxL,nameCont or "")
			end
		end
		local data = {}
		data.desc = name
		data.cont = localizedContinents[numCont] and numCont
		data.faction = (not pvpType or (pvpType == "friendly" or pvpType == "sanctuary")) and MYFACTION
		rawset(t,k,data)
		return data
	end})
	SlashCmdList["GATHERNOW"] = WTN.Slasher
	SLASH_GATHERNOW1 = "/gathernow"
end

function WTN.Slasher(option)
	local option = option and option:lower() or ""
	if option == "toggle" then
		GatherNow:SetShown(not GatherNow:IsShown())
	elseif option == "resetcache" then
		GatherNowDB.loc = wipe(GatherNowDB.loc or {})
		print("GatherNow: Cached zones wiped.")
	else -- unknown command; show help
		print("GatherNow: available commands")
		print("    /gathernow toggle")
		print("        (hides or shows the frame)")
		print("    /gathernow resetcache")
		print("        (clears cached zones; useful in the event of another Cataclysm or Blizzard adjusting zone levels)")
	end
end

function WTN:OnEnable()
	WTN:Create()
	print("GatherNow loaded succesfully. /gathernow for options")
end

function WTN:OnDisable()
	WTNframe:UnregisterEvent("CHAT_MSG_SKILL")
	WTNframe:UnregisterEvent("PLAYER_LOGOUT")
	WTNframe:UnregisterEvent("SKILL_LINES_CHANGED")
end
------------------------------------------------------------------------------

-- Registered Events -------------------------------------------------------
-- Handle skill up and update the display
WTN:RegisterEvent("CHAT_MSG_SKILL",
function()
	if isTracking then
		WTN:SkillTrackCreate(MsgTrack)
		fontstring2:SetFormattedText("%d/%d",skillLevel,maxSkillLevel)
		WTN:Location()
		WTN:SkillNowCheck()
	end
end)

WTN:RegisterEvent("SKILL_LINES_CHANGED",
function()
	if isTracking then
		WTN:SkillTrackCreate(MsgTrack)
		fontstring2:SetFormattedText("%d/%d",skillLevel,maxSkillLevel)
		WTN:Location()
		WTN:SkillNowCheck()
	end
end)
-- Save the screens postion on logout
WTN:RegisterEvent("PLAYER_LOGOUT",
function()
	GatherNowDB.point, GatherNowDB.relativeTo, GatherNowDB.relativePoint, GatherNowDB.x, GatherNowDB.y = WTNframe:GetPoint()
end)
------------------------------------------------------------------------------

--GatherNow Functions--------------------------------------------------------
-- Create the container on login
function WTN:Create()
	WTNframe = CreateFrame("Frame", "GatherNow", UIParent)
	WTNframe:SetWidth(125)
	WTNframe:SetHeight(80)
	WTNframe:SetFrameStrata("Background")
	if (GatherNowDB.point) then
		WTNframe:SetPoint(GatherNowDB.point, GatherNowDB.relativeTo, GatherNowDB.relativePoint, GatherNowDB.x, GatherNowDB.y)
	else
		WTNframe:SetPoint("CENTER", 0, 0)
	end

	WTNframe:EnableMouse(true)
	WTNframe:SetMovable(true)
	WTNframe:RegisterForDrag("LeftButton")

	local t = WTNframe:CreateTexture(nil,"BACKGROUND")
	t:SetTexture(EMPTY_TEXTURE)
	t:SetAllPoints(WTNframe)
	WTNframe.texture = t

	WTNframe:SetScript("OnDragStart",
	function(WTNframe, button)
		WTNframe:StartMoving()
	end)

	WTNframe:SetScript("OnDragStop",
	function(WTNframe, button)
		WTNframe:StopMovingOrSizing()
		-- get container position and save it here NYI
	end)

	fontstring = WTNframe:CreateFontString(nil,nil, "GameFontNormalSmall")
	fontstring2 = WTNframe:CreateFontString(nil,nil, "GameFontNormalSmall")
	fontstring3 = WTNframe:CreateFontString(nil,nil, "GameFontNormalSmall")
	fontstring2:SetFont(STANDARD_TEXT_FONT, 12, "")
	fontstring2:SetPoint("CENTER", 0, 30)

	fontstring3:SetFont(STANDARD_TEXT_FONT, 12, "")
	fontstring3:SetPoint("CENTER", 0, 30)
	fontstring3:SetText("Welcome to \n GatherNow.")
	WTN:CheckProfs()
	WTN:CacheContinents(GetMapContinents())
	WTN:CacheTrainerRanks()

	-- Create First Container button with default data and hide it
	bR1 = CreateFrame("Button", "GatherNowContainerbtn1", WTNframe)
	bR1:SetWidth(RBUTTON_WIDTH)
	bR1:SetHeight(RBUTTON_HEIGHT)
	bR1:SetPoint("CENTER", BUTTON1_HORZ_OFFSET, BUTTON1_VERT_OFFSET)
	teR1 = bR1:CreateTexture(nil,"BACKGROUND")
	bR1.texture = teR1
	teR1:SetTexture(EMPTY_TEXTURE)
	teR1:SetAllPoints(bR1)
	bR1.texture = teR1
	bR1.TOOLTIP_TITLE = ""
	bR1.TOOLTIP_TEXT = ""
	bR1:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(self.TOOLTIP_TITLE)
		GameTooltip:AddLine(self.TOOLTIP_TEXT,1,1,1)
		GameTooltip:Show()
	end)
	bR1:SetScript("OnLeave",GameTooltip_Hide)
	bR1.ClearData = function(bR1) -- create a function that allows us to 'reset' our button and store it on the button itself.
		bR1.TOOLTIP_TITLE = ""
		bR1.TOOLTIP_TEXT = ""
		bR1.texture:SetTexture(EMPTY_TEXTURE) 
	end

	-- Create Second Container button
	bR2 = CreateFrame("Button", "GatherNowContainerbtn2", WTNframe)
	bR2:SetWidth(RBUTTON_WIDTH)
	bR2:SetHeight(RBUTTON_HEIGHT)
	bR2:SetPoint("CENTER", BUTTON2_HORZ_OFFSET, BUTTON2_VERT_OFFSET)
	teR2 = bR2:CreateTexture(nil,"BACKGROUND")
	bR2.texture = teR2
	teR2:SetTexture(EMPTY_TEXTURE)
	teR2:SetAllPoints(bR2)
	bR2.texture = teR2
	bR2.TOOLTIP_TITLE = ""
	bR2.TOOLTIP_TEXT = ""
	bR2:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(self.TOOLTIP_TITLE)
		GameTooltip:AddLine(self.TOOLTIP_TEXT,1,1,1)
		GameTooltip:Show()
	end)
	bR2:SetScript("OnLeave",GameTooltip_Hide)
	bR2.ClearData = function(bR2) 
		bR2.TOOLTIP_TITLE = ""
		bR2.TOOLTIP_TEXT = ""
		bR2.texture:SetTexture(EMPTY_TEXTURE) 
	end
	
	-- Create Third and final container
	bR3 = CreateFrame("Button", "GatherNowContainerbtn3", WTNframe)
	bR3:SetWidth(RBUTTON_WIDTH)
	bR3:SetHeight(RBUTTON_HEIGHT)
	bR3:SetPoint("CENTER", BUTTON3_HORZ_OFFSET, BUTTON3_VERT_OFFSET)
	teR3 = bR3:CreateTexture(nil,"BACKGROUND")
	bR3.texture = teR3
	teR3:SetTexture(EMPTY_TEXTURE)
	teR3:SetAllPoints(bR3)
	bR3.texture = teR3
	bR3.TOOLTIP_TITLE = ""
	bR3.TOOLTIP_TEXT = ""
	bR3:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(self.TOOLTIP_TITLE)
		GameTooltip:AddLine(self.TOOLTIP_TEXT,1,1,1)
		GameTooltip:Show()
	end)
	bR3:SetScript("OnLeave",GameTooltip_Hide)
	bR3.ClearData = function(bR3) 
		bR3.TOOLTIP_TITLE = ""
		bR3.TOOLTIP_TEXT = ""
		bR3.texture:SetTexture(EMPTY_TEXTURE) 
	end

end

function WTN:CreateHerbButton()
	WTN:SkillTrackCreate(Herbalism)
	local b = CreateFrame("Button", "GatherNowHerbBtn", WTNframe)
	b:SetWidth(PBUTTON_WIDTH)
	b:SetHeight(PBUTTON_HEIGHT)
	b:SetPoint("CENTER", -75, 25)
	local te = b:CreateTexture(nil,"BACKGROUND")
	te:SetTexture(icon)
	te:SetAllPoints(b)
	b.texture = te
	b:RegisterForClicks("AnyUp", "AnyDown")
	b:SetScript("OnMouseUp",
	function(self, button)
		fontstring3:SetText(HERBALISM_NAME)
		isTracking = true
		TrackName = HERBALISM_NAME
		MsgTrack = Herbalism
		WTN:SkillTrackCreate(Herbalism)
		fontstring2:SetFormattedText("%d/%d",skillLevel,maxSkillLevel)
		WTN:Location()
	end)

end

function WTN:CreateMiningButton()
	WTN:SkillTrackCreate(Mining)
	local b1 = CreateFrame("Button", "GatherNowMiningBtn", WTNframe)
	b1:SetWidth(PBUTTON_WIDTH)
	b1:SetHeight(PBUTTON_HEIGHT)
	b1:SetPoint("CENTER", -75, 0)
	local te1 = b1:CreateTexture(nil,"BACKGROUND")
	te1:SetTexture(icon)
	te1:SetAllPoints(b1)
	b1.texture = te1
	b1:SetScript("OnMouseUp",
	function(self, button)
		fontstring3:SetText(MINING_NAME)
		isTracking =  true
		TrackName = MINING_NAME
		MsgTrack = Mining
		WTN:SkillTrackCreate(Mining)
		fontstring2:SetFormattedText("%d/%d",skillLevel,maxSkillLevel)
		WTN:Location()
	end)
end

-- the ellipsis ... is a special variable name used to hold an unknown number of arguments
-- select("#",...) gives us the number of arguments passed and allows us to traverse the list
function WTN:CacheContinents(...) 
	for i=1,select("#",...) do
		localizedContinents[i] = select(i,...)
	end
end

function WTN:CacheTrainerRanks()
	local numRanks = #PROFESSION_RANKS
	MAX_PROFESSION_SKILL = PROFESSION_RANKS[numRanks][1]
	-- store all the skills where a higher rank can be trained ordered from high to low.
	-- so we can later loop and compare with our own skill to the steps.
	-- as soon as we find a threshold we have passed (going down) we know we can train the skill at the previous iteration (ie the immediately higher)
	for rank=numRanks,1,-1 do
		local minTrainerSkill,maxTrainerSkill
		if rank > 1 then
			maxTrainerSkill = PROFESSION_RANKS[rank-1][1]
			minTrainerSkill = maxTrainerSkill - 25
			tinsert(localizedRankData,{minTrainerSkill,maxTrainerSkill,PROFESSION_RANKS[rank][2]}) -- eg. 500,525,"Zen Master"
		end
	end
end

function WTN:CheckProfs()
	prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
	-- Get Trade skill One
	MsgTrack = "" -- Clear data to prevent wrong skill info from displaying
	if (prof1) then
		WTN:SkillTrackCreate(prof1)
		if skillLine == HERBALISM_SKILLID then
			Herbalism = prof1
			HERBALISM_NAME = name
			WTN:CreateHerbButton()
		end
		if skillLine == MINING_SKILLID then
			Mining = prof1
			MINING_NAME = name
			WTN:CreateMiningButton()
		end
	end
	if (prof2) then
		WTN:SkillTrackCreate(prof2)
		if skillLine == HERBALISM_SKILLID then
			Herbalism = prof2
			HERBALISM_NAME = name
			WTN:CreateHerbButton()
		end
		if skillLine == MINING_SKILLID then
			Mining = prof2
			MINING_NAME = name
			WTN:CreateMiningButton()
		end
	end
	if not (Herbalism or Mining) then
		GatherNow:Hide()
	end
end

-- Get the professions information for displaying the skill level
function WTN:SkillTrackCreate(prof)
	if not tonumber(prof) then return end
	name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(prof)
end

-- see WTN:CacheContinents() for an explanation of the ellipsis variable ...
-- here we use it to pass an arbitrary number of mapIDs for which we want to
-- process the zones and spit out recommendations for the player.
function WTN:GetRecommendedAreas(...)
	-- zone = {desc="zone (l - l) continent",cont=continentID,faction="Alliance"|"Horde"}
	local numIDs = select("#",...) -- the number of zones we passed
	local output = ""
	for i=1,numIDs do
		-- select(i,...) will return the mapID at position i in the list of mapIDs we gave this function, eg. 30 if we passed the mapID for Elwynn Forest
		-- localizedMapData has our metatable magic so if the information for Elwynn is not already stored in our table
		-- the __index function of the metatable will try to get them from wow API, return them and store them in our SV for future lookups.
		local zone = localizedMapData[select(i,...)] 
		if (zone) then
			local description = zone.desc
			local continent = zone.cont
			local faction = zone.faction
			if (description) then
				if (not faction) or (not MYFACTION) or (MYFACTION == "Neutral") or (faction == MYFACTION) then
					if (not continent) or (not MYCONTINENT) or (continent == MYCONTINENT) then
						output = ("%s\n%s"):format(description,output) -- put same continent zones as the one we're on at the top
					else
						output = ("%s%s\n"):format(output,description) -- other continent zones at the bottom of the tooltip
					end
				end
			end
		end
	end
	if output ~= "" then
		output:gsub("\n$","") -- strip trailing newline if found
	end
	return output
end

-- Get the details of the resource that is being gathered
function WTN:GetResource(itemID)
	Rname, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemID)
	if not (Rname) then Rname = ("|cffff0000<%s>|r\nClick the Profession Icon to refresh."):format(RETRIEVING_ITEM_INFO); texture = GetItemIcon(itemID) end
end

-- check and Notify user to skill up. Used for all three professions
function WTN:SkillNowCheck()
	if skillLevel and maxSkillLevel and name then
		if skillLevel == maxSkillLevel then
			if MAX_PROFESSION_SKILL > maxSkillLevel then
				PlaySoundFile(DEFAULT_NOTIFY_SOUND, "Master")
				print(("You will not gain more skill in %s. Visit a trainer!"):format(name))
			end
		end
		-- this is where we traverse the stored trainer ranks from high to low
		-- until we find a range that contains our current skill
		-- then we prompt the user to train the one right above.
		-- data is a table with entries in the form of {1=minTrainSkill,2=maxTrainSkill,3=higherRankName}
		-- so data[1] can be 500, data[2] 525, data[3] "Zen Master"
		if skillLevel < maxSkillLevel then 
			for i,data in ipairs(localizedRankData) do
				if skillLevel >= data[1] and maxSkillLevel <= data[2] then
					PlaySoundFile(DEFAULT_NOTIFY_SOUND, "Master")
					print(("You can learn a new %s rank: %q!"):format(name,data[3]))
					break
				end
			end
		end
	end
end

-- Tell user closest and best place to mine
function WTN:Location()
	MYCONTINENT = GetCurrentMapContinent()
	fontstring3:SetFont(STANDARD_TEXT_FONT, 12, "")
	fontstring3:SetPoint("CENTER", 0, -13)

	-- 1. Clearing/reseting one of the 3 resource buttons is as simple as doing
	-- bRx:ClearData()
	-- 2. Setting the icon, tooltip title and text is also as simple as doing
	-- WTN:GetResource(itemID)
	-- bRx:TOOLTIP_TITLE = Rname
	-- bRx:TOOLTIP_TEXT = WTN:GetRecommendedAreas(mapID1,mapID2,etc)
	-- bRx.texture:SetTexture(texture)
	
	-- Account for start zones
	if isTracking then
		if TrackName == MINING_NAME then
			if skillLevel <= 49 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(2770)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544)
				-- "Elwynn Forest 1-10  \n Dun Morogh 1-10"
				-- "Teldrassil 1-10 \n Azuremyst Isle 1-10 \n Gilneas 5-12"
				-- "Tristfal Glades 1-10 \n Eversong Woods 1-10"
				-- "Durotar 1-10 \n Mulgore 1-10 \n The Lost Isles 5-12"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			-- Beyond startZone
			if skillLevel > 49 and skillLevel <= 99 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(2771)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(24,43) -- "Hillsbrad Foothills 20-25"-- "Ashenvale 20-25"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 99 and skillLevel <= 149 then
				bR1:ClearData()
				bR1:Show()

				WTN:GetResource(2772)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22,121) -- "Western Plaguelands 35-40"-- "Feralas 35-40"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 149 and skillLevel <= 199 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(3858)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(17,182) -- "Badlands 45-48"-- "Felwood 45-50"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 199 and skillLevel <= 274 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(10620)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19,281) -- "Blasted Lands 54-60"-- "Winterspring 50-55"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 274 and skillLevel <= 324 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(23424)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(465) -- "Hellfire Peninsula 58-63"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 324 and skillLevel <= 349 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(23425)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(477) -- "Nagrand 64-67"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 349 and skillLevel <= 399 then
				bR1:ClearData()
				bR1:Show()

				WTN:GetResource(36909)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(491,486) -- "Howling Fjord 68-72 \n Borean Tundra 68-72 "
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 399 and skillLevel <= 424 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(36912)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(493) -- "Sholzar Basin 76-78 "
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 424 and skillLevel <= 474 then
				bR1:ClearData()
				bR1:Show()
				-- Copper
				WTN:GetResource(53038)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(606) -- "Mount Hyjal 80-82"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 474 and skillLevel <= 499 then
				WTN:GetResource(52185)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(700,640) -- "Twilight Highlands 84-85 \n Deepholm 82-83"
				bR1.texture:SetTexture(texture)
				bR1:Show()

				WTN:GetResource(53038)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(640,606) -- "Deepholm 82-83 \n Mount Hyjal 80-82"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 499 and skillLevel <= 554 then
				WTN:GetResource(72092)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(806,857,807,809,858,811) -- "Jade Forest \nKrasarang Wilds \nValley of the Four Winds \nKun-Lai Summit \n Dread Waste \nVale of Eternal Blossom "
				bR1.texture:SetTexture(texture)
				bR1:Show()

				WTN:GetResource(72093)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(858,810,887) -- "Dread Wastes \nTownlong Steppes\nSiege of Niuzao Temple"
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end

		end
		if TrackName == HERBALISM_NAME then
			if skillLevel <= 14 then	-- Starter Zone farming
				-- Peacebloom
				WTN:GetResource(2447)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544) 
				-- "Elwynn Forest 1-10  \n Dun Morogh 1-10"
				-- "Teldrassil 1-10 \n Azuremyst Isle 1-10 \n Gilneas 5-12"
				-- "Tristfal Glades 1-10 \n Eversong Woods 1-10"
				-- "Durotar 1-10 \n Mulgore 1-10 \n The Lost Isles 5-12"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Silverleaf
				WTN:GetResource(765)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544)
				bR2.texture:SetTexture(texture)

				bR3:ClearData()
			end
			if skillLevel > 14 and skillLevel <= 49 then -- Start Zone w/ Earthroot available
				-- peacebloom
				WTN:GetResource(2447)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544)
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Silverleaf
				WTN:GetResource(765)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544)
				bR2.texture:SetTexture(texture)
				-- Earthroot
				WTN:GetResource(2449)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(30,27,41,464,684,20,462,4,9,544)
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 49 and skillLevel <= 69 then
				-- Earthroot
				WTN:GetResource(2449)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(39,36,35,30,27,42,476,41,464,684,21,463,20,462,11,181,4,9,544) 
				-- "Westfall 10-15 \n Redridge Mountains 15-20 \n Loch Modan 10-20 \n Elwynn Forest 1-10  \n Dun Morogh 1-10"
				-- "Darkshore 10-20 \n Bloodmyst Isle 11-20 \n Teldrassil 1-10 \n Azuremyst Isle 1-10 \n Gilneas 5-12"
				-- "Silverpine Foest 10-20\n Ghostlands 10-20 \n Trisfal Glades 1-10 \n Eversong Woods 1-10 "
				-- "Norhtern Barrens 10-20 \n Azshara 10-20 \n Durotar 1-10 \n Mulgore 1-10 \n The Lost Isles 5-12"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Mageroyal
				WTN:GetResource(785)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(39,36,35,42,476,21,463,11,181) 
				-- "Westfall 10-15 \n Redridge Mountains 15-20 \n Loch Modan 10-20"
				-- "Darkshore 10-20 \n Bloodmyst Isle 11-20 "
				-- "Silverpine Foest 10-20\n Ghostlands 10-20"
				-- "Norhtern Barrens 10-20 \n Azshara 10-20"
				bR2.texture:SetTexture(texture)

				-- Not Needed Empty Data
				bR3:ClearData()
			end
			if skillLevel > 69 and skillLevel <= 84 then
				-- Earthroot
				WTN:GetResource(2449)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(39,36,35,42,476,21,463,11,181)
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Mageroyal
				WTN:GetResource(785)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(39,36,35,42,476,21,463,11,181)
				bR2.texture:SetTexture(texture)
				-- Briarthorn
				WTN:GetResource(2450)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(39,36,35,42,476,21,463) 
				-- "Westfall 10-15 \n Redridge Mountains 15-20 \n Loch Modan 10-20"
				-- "Darkshore 10-20 \n Bloodmyst Isle 11-20 "
				-- "Silverpine Foest 10-20\n Ghostlands 10-20"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 84 and skillLevel <= 149 then
				-- Mageroyal
				WTN:GetResource(785)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(24,40,34,39,36,35,42,476,21,463,11,181) 
				-- "Hillsbrad Foothills 20-35 \n Wetlands 20-35 \n Duskwood 20-25 \n Westfall 10- 15 \n Redridge Mountains 15 - 20 \n Loch Modan 10-20"
				-- "Hillsbrad Foothills 20-35 \n Darkshore 10-20 \n Bloodmyst Isle 11-20"
				-- "Hillsbrad Foothills 20-35 \n Wetlands 20-35 \n Duskwood 20-25 \n Silverpine Foest 10-20\n Ghostlands 10-20"
				-- "Hillsbrad Foothills 20-35 \n Norhtern Barrens 10-20 \n Azshara 10-20"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Briarthorn
				WTN:GetResource(2450)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(24,39,36,35,42,476,21,463,11,181) 
				-- "Hillsbrad Foothills 20-35 \n Westfall 10-15 \n Redridge Mountains 15-20 \n Loch Modan 10-20"
				-- "Hillsbrad Foothills 20-35 \n Darkshore 10-20 \n Bloodmyst Isle 11-20 "
				-- "Hillsbrad Foothills 20-35 \n Silverpine Foest 10-20\n Ghostlands 10-20"
				-- "Hillsbrad Foothills 20-35 \n Norhtern Barrens 10-20 \n Azshara 10-20"
				bR2.texture:SetTexture(texture)
				-- Bruiseweed
				WTN:GetResource(2453)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(24,37,40,43,81) 
				-- "Hillsbrad Foothills 20-35 \n Northern Stranglethorn 25-30 \n Wetlands 20-25"
				-- "Hillsbrad Foothills 20-35 \n Ashenvale 20-25 \n Stonetalon Mountains 25-30"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 149 and skillLevel <= 159 then
				-- Liferoot
				WTN:GetResource(3357)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22) -- "Western Plaguelands 35-40"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- None
				bR2:ClearData()
				-- None
				bR3:ClearData()
			end
			if skillLevel > 159 and skillLevel <= 184 then
				-- Liferoot
				WTN:GetResource(3357)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22) -- "Western Plaguelands 35-40"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Fadeleaf
				WTN:GetResource(3818)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = EASTERN_KINGDOM_LOCATION1
				bR2.texture:SetTexture(texture)
				-- None
				bR3:ClearData()
			end
			if skillLevel > 184 and skillLevel <= 229 then
				-- Liferoot
				WTN:GetResource(3357)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22) -- "Western Plaguelands 35-40"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Fadeleaf
				WTN:GetResource(3818)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22) -- "Western Plaguelands 35-40"
				bR2.texture:SetTexture(texture)
				-- Khadgar's Whisker
				WTN:GetResource(3358)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(22) -- "Western Plaguelands 35-40"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 229 and skillLevel <= 259 then
				-- Khadgar's Whisker
				WTN:GetResource(3358)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(23) -- "Eastern Plaguelands 40-45"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Sungrass
				WTN:GetResource(8838)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(23) -- "Eastern Plaguelands 40-45"
				bR2.texture:SetTexture(texture)
				-- None
				bR3:ClearData()
			end
			if skillLevel > 259 and skillLevel <= 269 then
				-- Golden Sansam
				WTN:GetResource(13464)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- None

				bR2:ClearData()
				-- None

				bR3:ClearData()
			end
			if skillLevel > 269 and skillLevel <= 279 then
				-- Golden Sansam
				WTN:GetResource(13464)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Dreamfoil
				WTN:GetResource(13463)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR2.texture:SetTexture(texture)
				-- None

				bR3:ClearData()
			end
			if skillLevel > 279 and skillLevel <= 299 then
				-- Golden Sansam
				WTN:GetResource(13464)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Dreamfoil
				WTN:GetResource(13463)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR2.texture:SetTexture(texture)
				-- Mountain Silversage
				WTN:GetResource(13465)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(19) -- "Blasted Lands 54-60"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 299 and skillLevel <= 314 then
				-- Dreamfoil
				WTN:GetResource(13463)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(465) -- "Hellfire Peninsula 58-63"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Felweed
				WTN:GetResource(22785)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(465) -- "Hellfire Peninsula 58-63"
				bR2.texture:SetTexture(texture)
				-- Mountain Silversage
				bR3:ClearData()
			end
			if skillLevel > 314 and skillLevel <= 359 then
				-- Felweed
				WTN:GetResource(22785)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(465) -- "Hellfire Peninsula 58-63"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Dreaming Glory
				WTN:GetResource(22786)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(465) -- "Hellfire Peninsula 58-63"
				bR2.texture:SetTexture(texture)
				-- Mountain Silversage
				bR3:ClearData()
			end
			if skillLevel > 359 and skillLevel <= 374 then
				-- Goldclover
				WTN:GetResource(36901)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(491,486) -- "Howling Fjord 68-72 \n Borean Tundra 68-72"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- None
				bR2:ClearData()
				-- None
				bR3:ClearData()
			end
			if skillLevel > 374 and skillLevel <= 399  then
				-- Goldclover
				WTN:GetResource(36901)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(491,486) -- "Howling Fjord 68-72 \n Borean Tundra 68-72"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Tigerlilly
				WTN:GetResource(36904)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(491,486) -- "Howling Fjord 68-72 \n Borean Tundra 68-72"
				bR2.texture:SetTexture(Texture)
				-- None
				bR3:ClearData()
			end
			if skillLevel > 399 and skillLevel <= 449  then
				-- Goldclover
				WTN:GetResource(36901)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(493) -- "Sholzar Basin 76-78"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Tigerlilly
				WTN:GetResource(36904)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(493) -- "Sholzar Basin 76-78"
				bR2.texture:SetTexture(texture)
				-- Adder's Tongue
				WTN:GetResource(36903)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(493) -- "Sholzar Basin 76-78"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 449 and skillLevel <= 474  then
				-- Cinderbloom
				WTN:GetResource(52983)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(606) -- "Mount Hyjal 80-82"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Azshara's Veil
				WTN:GetResource(52985)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(606,613) -- EASTERN_KINGDOM_LOCATION1.." \n Vashj'ir 80-82"
				bR2.texture:SetTexture(texture)
				-- Stormvine
				WTN:GetResource(52984)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(606,613) -- EASTERN_KINGDOM_LOCATION1.." \n Vashj'ir 80-82"
				bR3.texture:SetTexture(texture)
			end
			if skillLevel > 474 and skillLevel <= 499  then
				-- Cinderbloom
				WTN:GetResource(52983)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(640) -- "Deepholm 82-83"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Heartblossom
				WTN:GetResource(52986)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(640) -- "Deepholm 82-83"
				bR2.texture:SetTexture(texture)
				-- Stormvine

				bR3:ClearData()
			end
			if skillLevel > 500 and skillLevel <= 600 then
				-- Green Tea Leaf
				WTN:GetResource(72234)
				bR1.TOOLTIP_TITLE = Rname
				bR1.TOOLTIP_TEXT = WTN:GetRecommendedAreas(806,807) -- "The Jade Forest\nValley of the Four Winds"
				bR1.texture:SetTexture(texture)
				bR1:Show()
				-- Rain Poppy
				WTN:GetResource(72237)
				bR2.TOOLTIP_TITLE = Rname
				bR2.TOOLTIP_TEXT = WTN:GetRecommendedAreas(806,811) -- "The Jade Forest\nVale of Eternal Blossoms"
				bR2.texture:SetTexture(texture)
				-- Silkweed
				WTN:GetResource(72235)
				bR3.TOOLTIP_TITLE = Rname
				bR3.TOOLTIP_TEXT = WTN:GetRecommendedAreas(807,857) -- "Valley of the Four winds\nKrasarang Wilds"
				bR3.texture:SetTexture(texture)
			end
		end
	end
end
